from enum import Enum
from typing import List

from pydantic import BaseModel

from watchmen_model.common import OptimisticLock, PluginId, TenantBasedTuple


class PluginType(str, Enum):
	STREAMLIT = 'streamlit',
	JUPYTER = 'jupyter',


class PluginApplyTo(str, Enum):
	ACHIEVEMENT = 'achievement'


class Plugin(TenantBasedTuple, OptimisticLock, BaseModel):
	pluginId: PluginId = None
	pluginCode: str = None
	name: str = None
	type: PluginType = None
	applyTo: PluginApplyTo = None
	# value is parameter name
	params: List[str] = None
	# value is result name
	results: List[str] = None



from typing import List

from watchmen_model.admin import Pipeline, Topic, PipelineTriggerType, PipelineStage, PipelineUnit, \
	WriteTopicActionType, Factor
from watchmen_model.admin.pipeline_action_write import InsertOrMergeRowAction, MappingFactor
from watchmen_model.common import TopicFactorParameter, ParameterKind, ParameterJoint, ParameterJointType, \
	ParameterExpression, ParameterExpressionOperator, ConstantParameter
from watchmen_utilities import ArrayHelper


def find_topic(topics: List[Topic], topic_name: str) -> Topic:
	return ArrayHelper(topics).find(lambda x: x.name == topic_name)



def find_factor(topic: Topic, factor_name: str) -> Factor:
	return ArrayHelper(topic.factors).find(lambda x: x.name == factor_name)


def find_source_parameter(topic: Topic, factor_name: str) -> TopicFactorParameter:
	return TopicFactorParameter(
		kind=ParameterKind.TOPIC, topicId=topic.topicId,
		factorId=find_factor(topic, factor_name).factorId
	)


# noinspection PyUnusedLocal
def ask_pipeline_monitor_pipelines(topics: List[Topic]) -> List[Pipeline]:
	topic_raw: Topic = find_topic(topics, 'raw_pipeline_monitor_log')
	topic_error: Topic = find_topic(topics, 'pipeline_monitor_error_log')

	return [
		Pipeline(
			name='Pipeline Error ',
			topicId=topic_raw.topicId,
			type=PipelineTriggerType.INSERT_OR_MERGE,
			stages=[PipelineStage(
				stageId='error-pipeline-s-1',
				name='Monitor Pipeline Stage 1',
				units=[PipelineUnit(
					unitId='error-u-1',
					name='error Pipeline Unit 1',
					do=[InsertOrMergeRowAction(
						actionId='error-a-1',
						type=WriteTopicActionType.INSERT_ROW,
						topicId=topic_error.topicId,
						mapping=[
							MappingFactor(
								source=find_source_parameter(topic_raw, 'dataId'),
								factorId=find_factor(topic_error, 'dataId').factorId),
							MappingFactor(
								source=find_source_parameter(topic_raw, 'topicId'),
								factorId=find_factor(topic_error, 'topicId').factorId),
							MappingFactor(
								source=find_source_parameter(topic_raw, 'status'),
								factorId=find_factor(topic_error, 'status').factorId),
							MappingFactor(
								source=find_source_parameter(topic_raw, 'traceId'),
								factorId=find_factor(topic_error, 'traceId').factorId),
							MappingFactor(
								source=find_source_parameter(topic_raw, 'pipelineId'),
								factorId=find_factor(topic_error, 'pipelineId').factorId),
							MappingFactor(
								source=find_source_parameter(topic_raw, 'uid'),
								factorId=find_factor(topic_error, 'uid').factorId),

						]
					)]
				)],
				conditional=True,
				on=ParameterJoint(
					jointType=ParameterJointType.AND,
					filters=[
						ParameterExpression(
							left=find_source_parameter(topic_raw, 'status'),
							operator=ParameterExpressionOperator.EQUALS,
							right=ConstantParameter(kind=ParameterKind.CONSTANT,value="ERROR")
						)
					]
				)
			)],
			enabled=True,
			validated=True
		)
	]



from datetime import date, datetime
from enum import Enum

from watchmen_model.common import Storable, SubscriptionEventId, SubscriptionEventLockId, TenantId, UserId


class JobLockStatus(str, Enum):
	READY = 'ready'
	FAILED = 'fail'
	SUCCESS = 'success'


class SubscriptionEventLock(Storable):
	subscriptionEventLockId: SubscriptionEventLockId = None
	tenantId: TenantId = None
	subscriptionEventId: SubscriptionEventId = None
	processDate: date = None
	status: JobLockStatus = None
	userId: UserId = None
	createdAt: datetime = None



from enum import Enum
from typing import List, Optional, Union

from pydantic import BaseModel

from watchmen_model.common import DataModel, NotificationDefinitionId, OptimisticLock, TenantBasedTuple, UserId
from watchmen_utilities import ArrayHelper


class NotificationParam(DataModel, BaseModel):
	name: str = None
	value: str = None


class NotificationType(str, Enum):
	EMAIL = "email"
	WEB_URL = "url"
	SLACK = "slack"
	FEISHU = "feishu"


def construct_param(param: Optional[Union[dict, NotificationParam]]) -> Optional[NotificationParam]:
	if param is None:
		return None
	elif isinstance(param, NotificationParam):
		return param
	else:
		return NotificationParam(**param)


def construct_params(params: Optional[list] = None) -> Optional[List[NotificationParam]]:
	if params is None:
		return None
	else:
		return ArrayHelper(params).map(lambda x: construct_param(x)).to_list()


class NotificationDefinition(TenantBasedTuple, OptimisticLock, BaseModel):
	notificationId: NotificationDefinitionId = None
	type: NotificationType = None
	params: List[NotificationParam] = []
	userId: UserId = None

	def __setattr__(self, name, value):
		if name == 'params':
			super().__setattr__(name, construct_params(value))
		else:
			super().__setattr__(name, value)



from enum import Enum
from typing import Optional

from pydantic import BaseModel

from watchmen_model.admin import TopicSnapshotFrequency
from watchmen_model.common import NotificationDefinitionId, OptimisticLock, SubscriptionEventId, TenantBasedTuple, \
	UserId
from watchmen_model.webhook.event_defination import EventSource


class ContentType(Enum):
	pass


class SubscriptionEvent(TenantBasedTuple, OptimisticLock, BaseModel):
	subscriptionEventId: SubscriptionEventId = None
	# eventId: EventDefinitionId = None
	eventCode: str = None
	eventSource: EventSource = None
	notificationId: NotificationDefinitionId = None
	sourceId: str = None
	userId: UserId = None
	contentType: ContentType = None
	# only for weekly
	weekday: Optional[str]
	# only for monthly
	day: Optional[str]
	hour: Optional[int] = None
	minute: Optional[int] = None
	enabled: bool = True
	status: bool = None
	frequency: TopicSnapshotFrequency = TopicSnapshotFrequency.DAILY

# TODO exception case monitor



from enum import Enum

from pydantic import BaseModel

from watchmen_model.admin import UserRole
from watchmen_model.common import EventDefinitionId, OptimisticLock, Tuple


class EventType(str, Enum):
	SYSTEM = 'system',
	BUSINESS = 'business'


class EventSource(str, Enum):
	SUBJECT = "subject"
	OBJECTIVE_ANALYSIS = "objective_analysis"


class EventDefinition(Tuple, OptimisticLock, BaseModel):
	eventDefinitionId: EventDefinitionId = None
	eventCode: str = None
	eventName: str = None
	eventType: EventType = None
	eventSource: EventSource = None
	role: UserRole = None



from __future__ import annotations

from enum import Enum
from typing import List, Optional, TypeVar, Union

from pydantic import BaseModel

from watchmen_model.common import BucketId, DataModel, FactorId, IndicatorId, ObjectiveFactorId, ObjectiveId, \
	ObjectiveTargetId, OptimisticLock, SubjectDatasetColumnId, TenantBasedTuple, UserGroupId
from watchmen_utilities import ArrayHelper

ObjectiveFactorName = TypeVar('ObjectiveFactorName', bound=str)


def construct_parameter_joint(
		joint: Optional[Union[dict, ObjectiveParameterJoint]]) -> Optional[ObjectiveParameterJoint]:
	if joint is None:
		return None
	elif isinstance(joint, ObjectiveParameterJoint):
		return joint
	else:
		# noinspection PyArgumentList
		return ObjectiveParameterJoint(**joint)


def construct_computed_parameter(
		parameter: Optional[Union[dict, ComputedObjectiveParameter]]) -> Optional[ComputedObjectiveParameter]:
	if parameter is None:
		return None
	elif isinstance(parameter, ComputedObjectiveParameter):
		return parameter
	else:
		# noinspection PyArgumentList
		return ComputedObjectiveParameter(**parameter)


def construct_parameter(parameter: Optional[Union[dict, ObjectiveParameter]]) -> Optional[ObjectiveParameter]:
	if parameter is None:
		return None
	elif isinstance(parameter, ObjectiveParameter):
		return parameter
	else:
		kind = parameter.get('kind')
		if kind == ObjectiveParameterType.REFER:
			return ReferObjectiveParameter(**parameter)
		elif kind == ObjectiveParameterType.CONSTANT:
			return ConstantObjectiveParameter(**parameter)
		elif kind == ObjectiveParameterType.COMPUTED:
			return ComputedObjectiveParameter(**parameter)
		elif kind == ObjectiveParameterType.BUCKET:
			return BucketObjectiveParameter(**parameter)
		elif kind == ObjectiveParameterType.TIME_FRAME:
			return TimeFrameObjectiveParameter(**parameter)
		else:
			raise Exception(f'Parameter kind[{kind}] cannot be recognized.')


def construct_parameters(parameters: Optional[list]) -> Optional[List[ObjectiveParameter]]:
	if parameters is None:
		return None
	return ArrayHelper(parameters).map(lambda x: construct_parameter(x)).to_list()


def construct_parameter_condition(
		condition: Optional[Union[dict, ObjectiveParameterCondition]]) -> Optional[ObjectiveParameterCondition]:
	if condition is None:
		return None
	elif isinstance(condition, ObjectiveParameterJoint):
		return condition
	elif isinstance(condition, ObjectiveParameterExpression):
		return condition
	else:
		joint_type = condition.get('jointType')
		if joint_type is None:
			return ObjectiveParameterExpression(**condition)
		else:
			return ObjectiveParameterJoint(**condition)


def construct_parameter_conditions(conditions: Optional[list]) -> Optional[List[ObjectiveParameterCondition]]:
	if conditions is None:
		return None
	return ArrayHelper(conditions).map(lambda x: construct_parameter_condition(x)).to_list()


class ObjectiveParameterType(str, Enum):
	REFER = 'refer',
	CONSTANT = 'constant',
	COMPUTED = 'computed',
	# only available on factor indicator filter
	BUCKET = 'bucket',
	# only available on factor indicator filter
	TIME_FRAME = 'time-frame'


# noinspection DuplicatedCode
class AvoidFastApiError:
	on: Optional[ObjectiveParameterJoint] = None


class ObjectiveParameter(DataModel, AvoidFastApiError, BaseModel):
	kind: ObjectiveParameterType = None
	conditional: bool = False

	def __setattr__(self, name, value):
		if name == 'on':
			super().__setattr__(name, construct_parameter_joint(value))
		else:
			super().__setattr__(name, value)


class ReferObjectiveParameter(ObjectiveParameter):
	"""
	it's a multiple purposes object.
	when it is used in factor/target formula, {@link #uuid} should refer to another objective factor.
	and when it is used in factor filter, {@link #uuid} should refer to factor from topic or column from subject dataset.
	"""
	kind: ObjectiveParameterType.REFER = ObjectiveParameterType.REFER
	uuid: Optional[Union[ObjectiveFactorId, FactorId, SubjectDatasetColumnId]] = None


class ConstantObjectiveParameter(ObjectiveParameter):
	kind: ObjectiveParameterType.CONSTANT = ObjectiveParameterType.CONSTANT
	value: Optional[str] = None


class ObjectiveFormulaOperator(str, Enum):
	NONE = 'none',
	ADD = 'add',
	SUBTRACT = 'subtract',
	MULTIPLY = 'multiply',
	DIVIDE = 'divide',
	MODULUS = 'modulus',
	YEAR_OF = 'year-of',
	HALF_YEAR_OF = 'half-year-of',
	QUARTER_OF = 'quarter-of',
	MONTH_OF = 'month-of',
	WEEK_OF_YEAR = 'week-of-year',
	WEEK_OF_MONTH = 'week-of-month',
	DAY_OF_MONTH = 'day-of-month',
	DAY_OF_WEEK = 'day-of-week',

	ROUND = 'round',
	FLOOR = 'floor',
	CEIL = 'ceil',
	ABS = 'abs',
	MAX = 'max',
	MIN = 'min',
	INTERPOLATE = 'interpolate',

	CASE_THEN = 'case-then',


# noinspection DuplicatedCode
class ComputedObjectiveParameter(ObjectiveParameter):
	kind: ObjectiveParameterType.COMPUTED = ObjectiveParameterType.COMPUTED
	operator: ObjectiveFormulaOperator = ObjectiveFormulaOperator.NONE
	parameters: List[ObjectiveParameter] = []

	def __setattr__(self, name, value):
		if name == 'parameters':
			super().__setattr__(name, construct_parameters(value))
		else:
			super().__setattr__(name, value)


class BucketObjectiveParameter(ObjectiveParameter):
	kind: ObjectiveParameterType.BUCKET = ObjectiveParameterType.BUCKET
	bucketId: Optional[BucketId] = None
	segmentName: Optional[str] = None


class TimeFrameObjectiveParameter(ObjectiveParameter):
	kind: ObjectiveParameterType.TIME_FRAME = ObjectiveParameterType.TIME_FRAME


class ObjectiveParameterCondition(DataModel, BaseModel):
	pass


# noinspection DuplicatedCode
class ObjectiveParameterExpressionOperator(str, Enum):
	EMPTY = 'empty',
	NOT_EMPTY = 'not-empty',
	EQUALS = 'equals',
	NOT_EQUALS = 'not-equals',
	LESS = 'less',
	LESS_EQUALS = 'less-equals',
	MORE = 'more',
	MORE_EQUALS = 'more-equals',
	IN = 'in',
	NOT_IN = 'not-in',


class ObjectiveParameterExpression(ObjectiveParameterCondition):
	left: Optional[ObjectiveParameter] = None
	operator: ObjectiveParameterExpressionOperator = ObjectiveParameterExpressionOperator.EQUALS
	right: Optional[ObjectiveParameter] = None

	def __setattr__(self, name, value):
		if name == 'left' or name == 'right':
			super().__setattr__(name, construct_parameter(value))
		else:
			super().__setattr__(name, value)


class ObjectiveParameterJointType(str, Enum):
	AND = 'and',
	OR = 'or',


class ObjectiveParameterJoint(ObjectiveParameterCondition):
	conj: ObjectiveParameterJointType = ObjectiveParameterJointType.AND
	filters: List[ObjectiveParameterCondition] = []

	def __setattr__(self, name, value):
		if name == 'filters':
			super().__setattr__(name, construct_parameter_conditions(value))
		else:
			super().__setattr__(name, value)


class ObjectiveTimeFrameKind(str, Enum):
	NONE = 'none',
	YEAR = 'year',
	HALF_YEAR = 'half-year',
	QUARTER = 'quarter',
	MONTH = 'month',
	WEEK_OF_YEAR = 'week-of-year',
	DAY_OF_MONTH = 'day-of-month',
	DAY_OF_WEEK = 'day-of-week',
	LAST_N_YEARS = 'last-n-years',
	LAST_N_MONTHS = 'last-n-months',
	LAST_N_WEEKS = 'last-n-weeks',
	LAST_N_DAYS = 'last-n-days'


class ObjectiveTimeFrameTill(str, Enum):
	NOW = 'now',
	LAST_COMPLETE_CYCLE = 'last-complete-cycle',
	SPECIFIED = 'specified'


class ObjectiveTimeFrame(DataModel, BaseModel):
	# is target in time frame, normally is
	kind: Optional[ObjectiveTimeFrameKind] = None
	# only available if kind is LAST_N-* types, should be a positive value
	lastN: str = None
	# time frame is cut off till when
	till: Optional[ObjectiveTimeFrameTill] = None
	# specify the till time when till is SPECIFIED
	specifiedTill: str = None


class ObjectiveTargetBetterSide(str, Enum):
	LESS = 'less',
	MORE = 'more'


def construct_asis(
		asis: Optional[Union[dict, ComputedObjectiveParameter, ObjectiveFactorId]]
) -> Optional[Union[ComputedObjectiveParameter, ObjectiveFactorId]]:
	if asis is None:
		return None
	elif isinstance(asis, str):
		return asis
	elif isinstance(asis, ComputedObjectiveParameter):
		return asis
	else:
		# noinspection PyArgumentList
		return ComputedObjectiveParameter(**asis)


class ObjectiveTarget(DataModel, BaseModel):
	uuid: ObjectiveTargetId = None

	name: Optional[str] = None
	# to be value, should be a numeric value, a percentage value
	tobe: Optional[str] = None
	# as is formula
	asis: Optional[Union[ComputedObjectiveParameter, ObjectiveFactorId]] = None
	# which side is better, with computed as is value vs to be value.
	betterSide: Optional[ObjectiveTargetBetterSide] = None
	# this July vs this June if time frame is on month, month-on-month
	askPreviousCycle: bool = False
	# this July vs last July if time frame is on month, year-on-year
	askChainCycle: bool = False

	def __setattr__(self, name, value):
		if name == 'asis':
			super().__setattr__(name, construct_asis(value))
		else:
			super().__setattr__(name, value)


class ObjectiveVariableKind(str, Enum):
	SINGLE_VALUE = 'value',
	BUCKET = 'bucket',
	RANGE = 'range'


class ObjectiveVariable(DataModel, BaseModel):
	name: str
	kind: ObjectiveVariableKind = None


class ObjectiveVariableOnValue(ObjectiveVariable):
	kind: ObjectiveVariableKind.SINGLE_VALUE = ObjectiveVariableKind.SINGLE_VALUE
	value: Optional[str] = None


class ObjectiveVariableOnBucket(ObjectiveVariable):
	kind: ObjectiveVariableKind.BUCKET = ObjectiveVariableKind.BUCKET
	bucketId: Optional[BucketId] = None
	segmentName: Optional[str] = None


class ObjectiveVariableOnRange(ObjectiveVariable):
	kind: ObjectiveVariableKind = ObjectiveVariableKind.RANGE
	min: Optional[str] = None
	includeMin: Optional[bool] = True
	max: Optional[str] = None
	includeMax: Optional[bool] = False


class ObjectiveFactorKind(str, Enum):
	INDICATOR = 'indicator',
	COMPUTED = 'computed'


class ObjectiveFactor(DataModel, BaseModel):
	uuid: ObjectiveFactorId = None
	kind: ObjectiveFactorKind = None
	name: ObjectiveFactorName = None
	formula: Optional[ComputedObjectiveParameter] = None

	def __setattr__(self, name, value):
		if name == 'formula':
			super().__setattr__(name, construct_computed_parameter(value))
		else:
			super().__setattr__(name, value)


class ObjectiveFactorOnIndicator(ObjectiveFactor):
	kind: ObjectiveFactorKind.INDICATOR = ObjectiveFactorKind.INDICATOR
	indicatorId: Optional[IndicatorId] = None
	conditional: bool = False
	# objective variables are available in constant value
	filter: Optional[ObjectiveParameterJoint] = None

	def __setattr__(self, name, value):
		if name == 'filter':
			super().__setattr__(name, construct_parameter_joint(value))
		else:
			super().__setattr__(name, value)


class ObjectiveFactorOnComputation(ObjectiveFactor):
	kind: ObjectiveFactorKind.COMPUTED = ObjectiveFactorKind.COMPUTED


def construct_time_frame(time_frame: Optional[Union[dict, ObjectiveTimeFrame]]) -> Optional[ObjectiveTimeFrame]:
	if time_frame is None:
		return None
	elif isinstance(time_frame, ObjectiveTimeFrame):
		return time_frame
	else:
		# noinspection PyArgumentList
		return ObjectiveTimeFrame(**time_frame)


def construct_target(target: Optional[Union[dict, ObjectiveTarget]]) -> Optional[ObjectiveTarget]:
	if target is None:
		return None
	elif isinstance(target, ObjectiveTarget):
		return target
	else:
		# noinspection PyArgumentList
		return ObjectiveTarget(**target)


def construct_targets(targets: Optional[list] = None) -> Optional[List[ObjectiveTarget]]:
	if targets is None:
		return None
	else:
		return ArrayHelper(targets).map(lambda x: construct_target(x)).to_list()


def construct_variable(variable: Optional[Union[dict, ObjectiveVariable]]) -> Optional[ObjectiveVariable]:
	if variable is None:
		return None
	elif isinstance(variable, ObjectiveVariable):
		return variable
	else:
		kind = variable.get('kind')
		if kind == ObjectiveVariableKind.SINGLE_VALUE:
			return ObjectiveVariableOnValue(**variable)
		elif kind == ObjectiveVariableKind.RANGE:
			return ObjectiveVariableOnRange(**variable)
		elif kind == ObjectiveVariableKind.BUCKET:
			return ObjectiveVariableOnBucket(**variable)
		else:
			raise Exception(f'Objective variable kind[{kind}] cannot be recognized.')


def construct_variables(variables: Optional[list] = None) -> Optional[List[ObjectiveVariable]]:
	if variables is None:
		return None
	else:
		return ArrayHelper(variables).map(lambda x: construct_variable(x)).to_list()


def construct_factor(factor: Optional[Union[dict, ObjectiveFactor]]) -> Optional[ObjectiveFactor]:
	if factor is None:
		return None
	elif isinstance(factor, ObjectiveFactorOnIndicator):
		return factor
	elif isinstance(factor, ObjectiveFactorOnComputation):
		return factor
	else:
		kind = factor.get('kind')
		if kind == ObjectiveFactorKind.INDICATOR:
			return ObjectiveFactorOnIndicator(**factor)
		elif kind == ObjectiveFactorKind.COMPUTED:
			return ObjectiveFactorOnComputation(**factor)
		else:
			raise Exception(f'Objective factor kind[{kind}] cannot be recognized.')


def construct_factors(factors: Optional[list] = None) -> Optional[List[ObjectiveFactor]]:
	if factors is None:
		return None
	else:
		return ArrayHelper(factors).map(lambda x: construct_factor(x)).to_list()


class Objective(TenantBasedTuple, OptimisticLock, BaseModel):
	objectiveId: ObjectiveId = None
	name: str = None
	description: str = None
	timeFrame: ObjectiveTimeFrame = None
	targets: List[ObjectiveTarget] = []
	variables: List[ObjectiveVariable] = []
	factors: List[ObjectiveFactor] = []
	groupIds: List[UserGroupId] = None

	def __setattr__(self, name, value):
		if name == 'timeFrame':
			super().__setattr__(name, construct_time_frame(value))
		elif name == 'targets':
			super().__setattr__(name, construct_targets(value))
		elif name == 'variables':
			super().__setattr__(name, construct_variables(value))
		elif name == 'factors':
			super().__setattr__(name, construct_factors(value))
		else:
			super().__setattr__(name, value)



from .achievement_plugin_task import AchievementPluginTask
from .bucket import Bucket, BucketSegment, BucketType, CategoryMeasureBucket, CategorySegment, CategorySegmentsHolder, \
	CategorySegmentValue, EnumMeasureBucket, MeasureBucket, NumericSegmentsHolder, NumericSegmentValue, \
	NumericValueBucket, NumericValueMeasureBucket, NumericValueSegment, OtherCategorySegmentValue, \
	RangeBucketValueIncluding
from .convergence import Convergence, CONVERGENCE_TARGET_VARIABLE_MAPPING_IGNORED, ConvergenceBucketVariable, \
	ConvergenceFreeWalkVariable, ConvergenceTarget, ConvergenceTargetVariableMapping, ConvergenceTimeFrameVariable, \
	ConvergenceTimeFrameVariableKind, ConvergenceVariable, ConvergenceVariableAxis, ConvergenceVariableType, \
	TimeFrameConvergenceVariableValue
from .derived_objective import BreakdownDimension, BreakdownDimensionType, BreakdownTarget, DerivedObjective
from .indicator import Indicator, IndicatorAggregateArithmetic, IndicatorBaseOn, IndicatorFilter, RelevantIndicator, \
	RelevantIndicatorType
from .measure_method import MeasureMethod
from .objective import BucketObjectiveParameter, ComputedObjectiveParameter, ConstantObjectiveParameter, Objective, \
	ObjectiveFactor, ObjectiveFactorKind, ObjectiveFactorName, ObjectiveFactorOnComputation, \
	ObjectiveFactorOnIndicator, ObjectiveFormulaOperator, ObjectiveParameter, ObjectiveParameterCondition, \
	ObjectiveParameterExpression, ObjectiveParameterExpressionOperator, ObjectiveParameterJoint, \
	ObjectiveParameterJointType, ObjectiveParameterType, ObjectiveTarget, ObjectiveTargetBetterSide, \
	ObjectiveTimeFrame, ObjectiveTimeFrameKind, ObjectiveTimeFrameTill, ObjectiveVariable, ObjectiveVariableKind, \
	ObjectiveVariableOnBucket, ObjectiveVariableOnRange, ObjectiveVariableOnValue, ReferObjectiveParameter, \
	TimeFrameObjectiveParameter



from .data_source import DataSource, DataSourceParam, DataSourceType
from .external_writer import ExternalWriter, ExternalWriterType
from .key_store import KeyStore
from .plugin import Plugin, PluginApplyTo, PluginType
from .tenant import Tenant
from .token import PersonalAccessToken, Token
from .operation import Operation, OperationType
from .PackageVersion import PackageVersion



from typing import List

from pydantic import BaseModel

from watchmen_model.common import ConvergenceId, ObjectiveId, OptimisticLock, SpaceId, TenantBasedTuple, UserGroupId, \
	UserId


class UserGroup(TenantBasedTuple, OptimisticLock, BaseModel):
	userGroupId: UserGroupId = None
	name: str = None
	description: str = None
	userIds: List[UserId] = None
	spaceIds: List[SpaceId] = None
	objectiveIds: List[ObjectiveId] = None
	convergenceIds: List[ConvergenceId] = None



from enum import Enum

from pydantic import BaseModel

from watchmen_model.common import AchievementPluginTaskId, Auditable, PluginId, UserBasedTuple


class AchievementPluginTaskStatus(str, Enum):
	SUBMITTED = 'submitted',
	SENT = 'sent',
	SUCCESS = 'success',
	FAILED = 'failed'


class AchievementPluginTask(UserBasedTuple, Auditable, BaseModel):
	achievementTaskId: AchievementPluginTaskId = None
	# TODO REFACTOR-OBJECTIVE ACHIEVEMENT IS DROPPED
	# achievementId: AchievementId = None
	achievementId: str = None
	pluginId: PluginId = None
	status: AchievementPluginTaskStatus = None
	url: str = None
